"""
validatorselecter Module

this contains the class which allows the ViewConfig to select the appropriate validator for the given field

Classes:
Name                               Description
MongoValidatorSelecter             Class to aid in the selection of a validator for a mongo document field

Exceptions:
None

Functions:
None


Copyright (c) 2009 Christopher Perkins
Original Version by Christopher Perkins 2009
Released under MIT license.

Mongo Contributions by Jorge Vargas
"""

from sprox.validatorselector import ValidatorSelector

class MongoKitValidatorSelector(ValidatorSelector):
    pass

