<table xmlns:py="http://genshi.edgewall.org/"
    class="${w.css_class}"
    py:attrs='w.attrs'>
<tr><th>Name</th><th>Definition</th></tr>
<py:for each="i, child in enumerate(w.children)">
${child.display(value=w.value, displays_on='genshi', css_class=((i%2 and 'even') or 'odd'))}
</py:for>
</table>
