<input xmlns="http://www.w3.org/1999/xhtml"
       xmlns:py="http://genshi.edgewall.org/"
       type="hidden" name="_method" class="${css_class}" id="${id}"
       value="PUT"
       py:attrs="attrs" />